#define EXTERN
#include "pglobalt.h"
#include "seqfilest.h"
#include "pspsolvert.h"
#include "pnpsolvert.h"

#include "input.h"
#include "output.h"
#include "globprep.h"

#include <stdio.h>
#include <string.h>
#include "mpi.h"



int main (int argc,char *argv[])
{
  XFILE *in;
  long err;
  descrip *d;

  fprintf (stderr,"\n\n *** PARTRFEL PREPROCESSOR ***\n");
  fprintf (stderr," -----------------------------\n");

  Tp  = new probdesct;
  Ptp = new pprobdesct;
  Tt  = new transtop;
  Gtt = new gtopology;
  Tm  = new transmat;
  Tc  = new transcrsec;
  Tb  = new transbclc;
  Outdt = new outdrivert;

  Ltt   = new trlineart;
  Ltat  = new trlinaxisym;
  Lqt   = new quadlineart;
  Lqat  = new quadlinaxisym;
  Ltett = new lintett;
  Lht   = new linhext;

  if (argc < 3){
    fprintf (stderr,"\n Wrong number of command line parameters.");
    fprintf (stderr,"\n Use : pmechprep input_file_name output_file name\n\n");
    delete Tp;  delete Tt;  delete Tm;  delete Tc;  delete Tb;  delete Ptp;
    return(1);
  }
  in = xfopen(argv[1],"r");
  if (in==NULL){
    fprintf (stderr,"\n Input file has not been specified.");
    fprintf (stderr,"\n Try it again!\n\n");
    delete Tp;  delete Tt;  delete Tm;  delete Tc;  delete Tb;  delete Ptp;
    return(2);
  }
  in->maxlnsize = 1024;
  in->warning = 1;
  in->kwdmode = ignore;
  //in->kwdmode = sequent_mode;
  in->ignorecase = 1;

  d = new descrip;
  memset (d->t3df,0,sizeof(*d->t3df)*1025);
  memset (d->matf,0,sizeof(*d->matf)*1025);
  memset (d->crf,0,sizeof(*d->crf)*1025);
  // reading line with topology file name
  xfscanf(in, "%a", d->t3df);
  // reading line with material database file name
  xfscanf(in, "%a", d->matf);
  // reading line with cross-section database file name
  xfscanf(in, "%a", d->crf);
  // reading line with topology file format indicator,
  xfscanf(in, "%ld", &d->t3d);
  d->seq = 0;
  // reading line with edge number indicator
  xfscanf(in, "%ld", &d->redgn);
  // reading problem description
  in->kwdmode = sequent_mode;
  xfscanf(in, "%k", "begin");
  in->kwdmode = ignore;
  
  //  creation of object of parallel solver
  Psolt = new psolver (Nproc,Myrank,NULL,0);
  xfscanf (in,"%ld",&Ndom);
  Tp->read(in);
  Psolt->read (in,Gtt,Ndom,Tp->tstorkm,Mesprt);

  in->kwdmode = sequent_mode;
  xfscanf(in, "%k", "end");
  in->kwdmode = ignore;

  err = input(in, d);
  if(err)
  {
    xfclose(in);
    fprintf (stderr,"\n\n");
    fprintf (stdout,"\n\n");
    return(err);
  }

  // reading output description
  in->kwdmode = sequent_mode;
  xfscanf(in, "%k", "begin");
  in->kwdmode = ignore;

  Outdt->read(in);
  xfclose(in);

  FILE *out = fopen(argv[2], "wt");
  if (out==NULL){
    fprintf (stderr,"\n Output file couldn't be opened.");
    fprintf (stderr,"\n Try it again!\n\n");
    fprintf (stderr,"\n\n");
    fprintf (stdout,"\n\n");
    return(7);
  }

  fprintf(stdout, "\nGeneral topology export . . .");
  Top->exporttop(Gtt);
  fprintf(stdout, " O.K.\n");

  fprintf(stdout, "\nProblem description output . . .");
  fprintf (out,"%ld\n",Ndom);
  Tp->print(out);
  Psolt->print (out);
  fprintf(stdout, " O.K.\n");
 
  fprintf(stdout, "\nTransport topology export . . .");
  Tt->alloc_prep(Top->nn, Top->ne);
  fprintf(stdout, " O.K.\n");

  output(out, d);

  fprintf(stdout, "\nOutdriver description output . . .");
  Outdt->print(out);
  fprintf(stdout, " O.K.\n");

  fclose(out);

  fprintf (stderr,"\n\n");
  fprintf (stdout,"\n\n");
  delete d;
  //delete Ptp; delete Tp;  delete Tt;  delete Gtt;  delete Tm;  delete Tc;  delete Tb;
  return(0);
}
